@@ -325,11 +325,14 @@ static ssize_t __bufp_recvmsg(struct rtipc_private *priv,
 		if (iov[nvec].iov_len == 0)
 			continue;
 		vlen = wrlen >= iov[nvec].iov_len ? iov[nvec].iov_len : wrlen;
+#ifdef CONFIG_XENO_OPT_PERVASIVE
 		if (user_info) {
 			xnbufd_map_uread(&bufd, iov[nvec].iov_base, vlen);
 			ret = __bufp_readbuf(sk, &bufd, flags);
 			xnbufd_unmap_uread(&bufd);
-		} else {
+		} else
+#endif
+		{
 			xnbufd_map_kread(&bufd, iov[nvec].iov_base, vlen);
 			ret = __bufp_readbuf(sk, &bufd, flags);
 			xnbufd_unmap_kread(&bufd);
@@ -575,11 +578,14 @@ static ssize_t __bufp_sendmsg(struct rtipc_private *priv,
 		if (iov[nvec].iov_len == 0)
 			continue;
 		vlen = rdlen >= iov[nvec].iov_len ? iov[nvec].iov_len : rdlen;
+#ifdef CONFIG_XENO_OPT_PERVASIVE
 		if (user_info) {
 			xnbufd_map_uread(&bufd, iov[nvec].iov_base, vlen);
 			ret = __bufp_writebuf(rsk, sk, &bufd, flags);
 			xnbufd_unmap_uread(&bufd);
-		} else {
+		} else
+#endif
+		{
 			xnbufd_map_kread(&bufd, iov[nvec].iov_base, vlen);
 			ret = __bufp_writebuf(rsk, sk, &bufd, flags);
 			xnbufd_unmap_kread(&bufd);
