@@ -315,11 +315,14 @@ static ssize_t __xddp_recvmsg(struct rtipc_private *priv,
 		if (iov[nvec].iov_len == 0)
 			continue;
 		vlen = wrlen >= iov[nvec].iov_len ? iov[nvec].iov_len : wrlen;
+#ifdef CONFIG_XENO_OPT_PERVASIVE
 		if (user_info) {
 			xnbufd_map_uread(&bufd, iov[nvec].iov_base, vlen);
 			ret = xnbufd_copy_from_kmem(&bufd, mbuf->data + rdoff, vlen);
 			xnbufd_unmap_uread(&bufd);
-		} else {
+		} else
+#endif
+		{
 			xnbufd_map_kread(&bufd, iov[nvec].iov_base, vlen);
 			ret = xnbufd_copy_from_kmem(&bufd, mbuf->data + rdoff, vlen);
 			xnbufd_unmap_kread(&bufd);
@@ -512,11 +515,14 @@ static ssize_t __xddp_sendmsg(struct rtipc_private *priv,
 			if (iov[nvec].iov_len == 0)
 				continue;
 			vlen = rdlen >= iov[nvec].iov_len ? iov[nvec].iov_len : rdlen;
+#ifdef CONFIG_XENO_OPT_PERVASIVE
 			if (user_info) {
 				xnbufd_map_uread(&bufd, iov[nvec].iov_base, vlen);
 				ret = __xddp_stream(rsk, from, &bufd);
 				xnbufd_unmap_uread(&bufd);
-			} else {
+			} else
+#endif
+			{
 				xnbufd_map_kread(&bufd, iov[nvec].iov_base, vlen);
 				ret = __xddp_stream(rsk, from, &bufd);
 				xnbufd_unmap_kread(&bufd);
@@ -555,11 +561,14 @@ nostream:
 		if (iov[nvec].iov_len == 0)
 			continue;
 		vlen = rdlen >= iov[nvec].iov_len ? iov[nvec].iov_len : rdlen;
+#ifdef CONFIG_XENO_OPT_PERVASIVE
 		if (user_info) {
 			xnbufd_map_uread(&bufd, iov[nvec].iov_base, vlen);
 			ret = xnbufd_copy_to_kmem(mbuf->data + wrlen, &bufd, vlen);
 			xnbufd_unmap_uread(&bufd);
-		} else {
+		} else
+#endif
+		{
 			xnbufd_map_kread(&bufd, iov[nvec].iov_base, vlen);
 			ret = xnbufd_copy_to_kmem(mbuf->data + wrlen, &bufd, vlen);
 			xnbufd_unmap_kread(&bufd);
